@@ -319,7 +319,7 @@ void EditorExtension::insertIntelligentNewline(Kate::View *view)
 
 	if ( findOpenedEnvironment(row,col,name,view) )
 	{
-		kdDebug() << "   name==" << name << " " << m_latexCommands->isListEnv(name) << endl;
+		//kdDebug() << "   name==" << name << " " << m_latexCommands->isListEnv(name) << endl;
 		if ( m_latexCommands->isListEnv(name) )
 		{
 			view->keyReturn();
@@ -590,7 +590,7 @@ bool EditorExtension::findEndEnvironment(Kate::Document *doc, uint row, uint col
 bool EditorExtension::findEnvironmentTag(Kate::Document *doc, uint row, uint col,
                                   EnvData &env,bool backwards)
 {
-	kdDebug() << "findEnvTag " << backwards << endl;
+	//kdDebug() << "findEnvTag " << backwards << endl;
 	KTextEditor::SearchInterface *iface;
 	iface = dynamic_cast<KTextEditor::SearchInterface *>(doc);
 	
@@ -697,7 +697,7 @@ bool EditorExtension::isEnvironmentPosition(Kate::Document *doc, uint row, uint
 		}
 	}
 	
-		kdDebug() << "search right " << endl;
+	//kdDebug() << "search right " << endl;
 	// check if there is a match in this line from the current position to the right
 	if ( textline[col]=='\\' && col==(uint)textline.find(m_reg,col) )
 	{
@@ -707,20 +707,20 @@ bool EditorExtension::isEnvironmentPosition(Kate::Document *doc, uint row, uint
 		QChar ch = textline.at(col+1);
 		if ( ch=='b' || ch=='e' )        // found "\begin" or "\end"
 		{
-			env.tag = ( ch == 'b' ) ? EnvBegin : EnvEnd;
-			env.name = m_reg.cap(3);
+			envright.tag = ( ch == 'b' ) ? EnvBegin : EnvEnd;
+			envright.name = m_reg.cap(3);
 		} 
 		else                             // found "\[" or "\\]"
 		{
-			env.tag = ( ch == '[' ) ? EnvBegin : EnvEnd;
-			env.name = m_reg.cap(4);
+			envright.tag = ( ch == '[' ) ? EnvBegin : EnvEnd;
+			envright.name = m_reg.cap(4);
 		}
 		envright.cpos = EnvLeft;
 		right = true;
 		//kdDebug() << "   is - found right:  pos=" <<col << " " << envright.name << " " << QString(textline.at(col+1)) << endl;
 	}
 	
-		kdDebug() << "found left/right: " << left << "/" << right << endl;
+	//kdDebug() << "found left/right: " << left << "/" << right << endl;
 	// did we find a tag?
 	if ( ! (left || right) )
 		return false;
@@ -879,7 +879,7 @@ bool EditorExtension::decreaseCursorPosition(Kate::Document *doc, uint &row, uin
 	else if ( row > 0 )
 	{
 		--row;
-		col = doc->lineLength(row) - 1;
+		col = doc->lineLength(row);
 	}
 	else
 		ok = false;
@@ -1109,7 +1109,7 @@ bool EditorExtension::isBracketPosition(Kate::Document *doc, uint row, uint col,
 	QChar right = textline[col];
 	QChar left  = ( col > 0 ) ? textline[col-1] : QChar(' ');
 	
-	kdDebug() << QString("isBracketPosition: (%1,%2) left %3 right %4").arg(row).arg(col).arg(left).arg(right) << endl;
+	//kdDebug() << QString("isBracketPosition: (%1,%2) left %3 right %4").arg(row).arg(col).arg(left).arg(right) << endl;
 	if ( m_overwritemode )
 	{
 		if ( right == '{' )
@@ -1191,7 +1191,7 @@ bool EditorExtension::findOpenBracketTag(Kate::Document *doc, uint row, uint col
 		int start = ( line == (int)row ) ? col : textline.length()-1;
 		for ( int i=start; i>=0; --i )
 		{
-			kdDebug() << "findOpenBracketTag: (" << line << "," << i << ") = " << textline[i].latin1() << endl;
+			//kdDebug() << "findOpenBracketTag: (" << line << "," << i << ") = " << textline[i].latin1() << endl;
 			if ( textline[i] == '{' )
 			{
 				if ( brackets > 0 )
@@ -1211,7 +1211,7 @@ bool EditorExtension::findOpenBracketTag(Kate::Document *doc, uint row, uint col
 		}
 	}
 	
-	kdDebug() << "nothting found" << endl;
+	//kdDebug() << "nothting found" << endl;
 	return false;
 }
 
@@ -1541,7 +1541,16 @@ bool EditorExtension::insertDoubleQuotes()
 	uint row,col;
 	view->cursorPositionReal(&row,&col);
 	Kate::Document *doc = view->getDoc();
-	
+	if(doc)
+		doc->removeSelectedText();
+
+	// simply insert, if we are inside a verb command
+	if ( insideVerb(view) || insideVerbatim(view) )
+	{
+		doc->insertText(row,col,"\"");
+		return true;
+	}
+
 	// simply insert, if autoinsert mode is not active or the char bevor is \ (typically for \"a useful)
 	if ( !m_dblQuotes || ( col > 0 && doc->text(row,col-1,row,col) == QString("\\") ) ) 
 	{
@@ -1567,18 +1576,18 @@ bool EditorExtension::insertDoubleQuotes()
 	if ( iface->searchText(row,col,reg,&r,&c,&l,true) )  
 	{
 		openfound = ( doc->textLine(r).find(m_leftDblQuote,c) == (int)c );
-		kdDebug() << "pattern=" << reg.pattern() << " " << reg.cap(1) << " r=" << r << " c=" << c << " open=" << openfound<< endl;
+		//kdDebug() << "pattern=" << reg.pattern() << " " << reg.cap(1) << " r=" << r << " c=" << c << " open=" << openfound<< endl;
 	}
 	
 	QString textline = doc->textLine(row);
-	kdDebug() << "text=" << textline << " open=" << openfound << endl;
+	//kdDebug() << "text=" << textline << " open=" << openfound << endl;
 	if ( openfound ) 
 	{
 		// If we last inserted a language specific doublequote open,  
 		// we have to change it to a normal doublequote. If not we 
 		// insert a language specific doublequote close
 		int startcol = col - m_leftDblQuote.length();
-		kdDebug() << "startcol=" << startcol << " col=" << col  << endl;
+		//kdDebug() << "startcol=" << startcol << " col=" << col  << endl;
 		if ( startcol>=0 && textline.find(m_leftDblQuote,startcol) == (int)startcol ) 
 		{
 				doc->removeText(row,startcol,row,startcol+m_leftDblQuote.length());
@@ -1595,7 +1604,7 @@ bool EditorExtension::insertDoubleQuotes()
 		// we have to change it to a normal doublequote. If not we 
 		// insert a language specific doublequote open
 		int startcol = col - m_rightDblQuote.length();
-		kdDebug() << "startcol=" << startcol << " col=" << col  << endl;
+		//kdDebug() << "startcol=" << startcol << " col=" << col  << endl;
 		if ( startcol>=0 && textline.find(m_rightDblQuote,startcol) == (int)startcol ) 
 		{
 			doc->removeText(row,startcol,row,startcol+m_rightDblQuote.length());
@@ -1693,35 +1702,40 @@ bool EditorExtension::eventInsertEnvironment(Kate::View *view)
 
 bool EditorExtension::shouldCompleteEnv(const QString &env, Kate::View *view)
 {
-	kdDebug() << "===EditorExtension::shouldCompleteEnv(...)===" << endl;
- 	QString envname = env;
-	envname.replace("*","\\*");
+	kdDebug() << "===EditorExtension::shouldCompleteEnv( " << env << " )===" << endl;
 	QRegExp reTestBegin,reTestEnd;
-	if ( envname == "\\[" )
+	if ( env == "\\[" )
 	{
-		reTestBegin.setPattern("\\\\\\[");
-		reTestEnd.setPattern("\\\\\\]");
+		kdDebug() << "display style" << endl;
+		reTestBegin.setPattern("(?:[^\\\\]|^)\\\\\\["); 
+		// the first part is a non-capturing bracket (?:...) and we check if we don't have a backslash in front,
+		//  or that we are at the begin of the line
+		reTestEnd.setPattern("(?:[^\\\\]|^)\\\\\\]");
 	}
 	else
 	{
-		reTestBegin.setPattern("\\\\begin\\s*\\{" + envname + "\\}");
-		reTestEnd.setPattern("\\\\end\\s*\\{" + envname + "\\}");
+		reTestBegin.setPattern("(?:[^\\\\]|^)\\\\begin\\s*\\{" + QRegExp::escape(env) + "\\}");
+		reTestEnd.setPattern("(?:[^\\\\]|^)\\\\end\\s*\\{" + QRegExp::escape(env) + "\\}");
 	}
-	
+
 	int num = view->getDoc()->numLines();
 	int numBeginsFound = 0;
 	int numEndsFound = 0;
 	uint realLine, realColumn;
-	view->cursorPositionReal(&realLine, &realColumn);
+	view->cursorPositionReal(&realLine, &realColumn);	
 	for ( int i = realLine; i < num; ++i)
 	{
 		numBeginsFound += view->getDoc()->textLine(i).contains(reTestBegin);
 		numEndsFound += view->getDoc()->textLine(i).contains(reTestEnd);
-		if ( (numBeginsFound == 1) && (numEndsFound == 1) ) return false;        
-		else if ( (numEndsFound == 0) && (numBeginsFound > 1) ) return true;
-		else if ( (numBeginsFound > 2) || (numEndsFound > 1) ) return true; //terminate the search
+		kdDebug() << "line is " << i <<  " numBeginsFound = " << numBeginsFound <<  " , " << "numEndsFound = " << numEndsFound << endl;
+		if ( numEndsFound >= numBeginsFound )
+			return false;
+		else if ( numEndsFound == 0 && numBeginsFound > 1 )
+			return true;
+		else if ( numBeginsFound > 2 || numEndsFound > 1 )
+			return true; // terminate the search
 	}
-    
+
 	return true;
 }
 
@@ -1736,6 +1750,48 @@ QString EditorExtension::getWhiteSpace(const QString &s)
 	return whitespace;
 }
 
+//////////////////// inside verbatim commands ////////////////////
+
+bool EditorExtension::insideVerbatim(Kate::View *view)
+{
+	uint rowEnv,colEnv;
+	QString nameEnv;
+
+	if ( findOpenedEnvironment(rowEnv,colEnv,nameEnv,view) )
+	{
+		if ( m_latexCommands->isVerbatimEnv(nameEnv) )
+			return true;
+	}
+
+	return false;
+}
+
+bool EditorExtension::insideVerb(Kate::View *view)
+{
+	view = determineView(view);
+	if ( !view ) return false;
+	
+	// get current position
+	uint row,col;
+	view->cursorPositionReal(&row,&col);
+
+	int startpos = 0;
+	QString textline = getTextLineReal(view->getDoc(),row);
+	QRegExp reg("\\\\verb(\\*?)(.)");
+	while ( true )
+	{
+		int pos = textline.find(reg,startpos);
+		if ( pos<0 || col<(uint)pos+6+reg.cap(1).length() ) 
+			return false; 
+
+		pos = textline.find(reg.cap(2),pos+6+reg.cap(1).length());
+		if ( pos<0 || col<=(uint)pos )
+			return true;
+
+		startpos = pos + 1;
+	}
+}
+
 }
 
 #include "kileedit.moc"
